.. -*- mode: rst -*-

| 

|bfieldtools|_

|

|GH-Actions|_ |Codecov|_ |Zenodo|_ |pypi|_ 

.. |bfieldtools| image:: https://bfieldtools.github.io/_static/logo_simple_w_name.svg
.. _bfieldtools: https://bfieldtools.github.io

.. |GH-Actions| image:: https://github.com/bfieldtools/bfieldtools/workflows/Run%20tests/badge.svg
.. _GH-Actions: https://github.com/bfieldtools/bfieldtools/actions

.. |Codecov| image:: https://codecov.io/gh/bfieldtools/bfieldtools/branch/master/graph/badge.svg?token=R6WGNQ4QBV
.. _Codecov: https://codecov.io/gh/bfieldtools/bfieldtools

.. |pypi| image:: https://badge.fury.io/py/bfieldtools.svg
.. _pypi: https://pypi.org/project/bfieldtools/

.. |Zenodo| image:: https://zenodo.org/badge/255892837.svg
.. _Zenodo: https://zenodo.org/badge/latestdoi/255892837


bfieldtools is a software package for magnetic field modeling with an emphasis on magnetic field generated by a continuous surface-current density. The surface-current density is described by a piecewise linear stream function on a triangle mesh.

The software currently features
 - Discrete calculus on triangle meshes, including the Laplacian, gradient, divergence and curl
 - Mesh operators (matrices) for inductance and resistance
 - Magnetic field as well as scalar and vector potentials of surface-current densities
 - Magnetic field as well as scalar and vector potentials of polyline currents
 - Spherical harmonics (multipole) representation of the magnetic field
 - Surface harmonics (Laplace-Beltrami eigenfunctions) for series expansion of the stream function
 - Coil design using convex optimization
 - Magnetic thermal noise of thin conductors

Dependencies
^^^^^^^^^^^^

Requires Python 3.6 or newer.

 - `Numpy <https://www.numpy.org/>`_
 - `Scipy <https://www.scipy.org/>`_
 - `Trimesh <https://github.com/mikedh/trimesh>`_
 - `CVXPY <https://cvxpy.org/>`_
 - `Matplotlib <https://matplotlib.org/>`_
 - `Mayavi <https://docs.enthought.com/mayavi/mayavi/>`_

Documentation
^^^^^^^^^^^^^

Online documentation, including an API reference and examples can be found at https://bfieldtools.github.io/.


Installing bfieldtools
^^^^^^^^^^^^^^^^^^^^^^

To install bfieldtools using pip, simply run:
    
.. code-block:: bash

    pip install bfieldtools
    
    
For advanced installation instructions, see the online `installation instructions`_.

.. _installation instructions: https://bfieldtools.github.io/installation.html


Citing bfieldtools
^^^^^^^^^^^^^^^^^^

When using the software package in a publication, please cite the following articles:

.. code-block:: BiBTeX


	@article{doi:10.1063/5.0016090,
		author = {Mäkinen, Antti J. and Zetter, Rasmus and Iivanainen, Joonas and Zevenhoven, Koos C. J. and Parkkonen, Lauri and Ilmoniemi, Risto J.},
		title = {Magnetic-field modeling with surface currents. Part I. Physical and computational principles of bfieldtools},
		journal = {Journal of Applied Physics},
		volume = {128},
		number = {6},
		pages = {063906},
		year = {2020},
		doi = {10.1063/5.0016090},
		URL = {https://doi.org/10.1063/5.0016090},
		eprint = {https://doi.org/10.1063/5.0016090}
	}	


	@article{doi:10.1063/5.0016087,
		author = {Zetter, Rasmus and Mäkinen, Antti J. and Iivanainen, Joonas and Zevenhoven, Koos C. J. and Ilmoniemi, Risto J. and Parkkonen, Lauri},
		title = {Magnetic field modeling with surface currents. Part II. Implementation and usage of bfieldtools},
		journal = {Journal of Applied Physics},
		volume = {128},
		number = {6},
		pages = {063905},
		year = {2020},
		doi = {10.1063/5.0016087},
		URL = {https://doi.org/10.1063/5.0016087},
		eprint = {https://doi.org/10.1063/5.0016087}
	}    
    
When using the thermal noise computations, please cite this `preprint <https://arxiv.org/abs/2007.08963>`__ in arXiv.

Contributing
^^^^^^^^^^^^

bfieldtools is under active development, and contributions are warmly welcome! For new functionality, please write tests to accompany the code! bfieldtools uses Black_ for code style enforcement, please run Black on any new code.

In general, The MNE-Python project provides `an overview of good practices`_ that (when applicable) should be followed in bfieldtools as well.

.. _Black: https://github.com/psf/black
.. _an overview of good practices: https://mne.tools/dev/install/contributing.html

License
^^^^^^^

bfieldtools is **BSD-licenced** (3 clause):

    Copyright (c) 2019, authors of bfieldtools.
    All rights reserved.

    Redistribution and use in source and binary forms, with or without
    modification, are permitted provided that the following conditions are met:

    * Redistributions of source code must retain the above copyright notice,
      this list of conditions and the following disclaimer.

    * Redistributions in binary form must reproduce the above copyright notice,
      this list of conditions and the following disclaimer in the documentation
      and/or other materials provided with the distribution.

    * Neither the names of bfieldtools authors nor the names of any
      contributors may be used to endorse or promote products derived from
      this software without specific prior written permission.

    **This software is provided by the copyright holders and contributors
    "as is" and any express or implied warranties, including, but not
    limited to, the implied warranties of merchantability and fitness for
    a particular purpose are disclaimed. In no event shall the copyright
    owner or contributors be liable for any direct, indirect, incidental,
    special, exemplary, or consequential damages (including, but not
    limited to, procurement of substitute goods or services; loss of use,
    data, or profits; or business interruption) however caused and on any
    theory of liability, whether in contract, strict liability, or tort
    (including negligence or otherwise) arising in any way out of the use
    of this software, even if advised of the possibility of such
    damage.**



